﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using OpenTK;
using OpenTK.Graphics.OpenGL;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

namespace OpenTK_Test
{
	class ContentPipe
	{
		/// <summary>
		/// 지정한 이미지를 텍스처로 만듭니다.
		/// </summary>
		/// <param name="path"></param>
		/// <returns></returns>
		public static Texture2D LoadTexture(string path)
		{
			if (!File.Exists("Content/" + path))
			{
				throw new FileNotFoundException("파일을 찾을 수 없습니다. '" + path + "'");
			}
			
			//텍스처를 생성하고 연결된 텍스처의 id를 받아옵니다.
			int id = GL.GenTexture();
			//2D 텍스처임을 알린다.
			GL.BindTexture(TextureTarget.Texture2D, id);

			//이미지를 비트맵형식으로 불러온다.
			Bitmap bmp = new Bitmap("Content/" + path);
			BitmapData data = bmp.LockBits(new Rectangle(0, 0, bmp.Width, bmp.Height)
											, ImageLockMode.ReadOnly
											, System.Drawing.Imaging.PixelFormat.Format32bppArgb);

			//텍스처를 만들어 마지막으로 생성한 텍스처id에 저장합니다.
			GL.TexImage2D(TextureTarget.Texture2D
							, 0
							, PixelInternalFormat.Rgba
							, data.Width
							, data.Height
							, 0
							, OpenTK.Graphics.OpenGL.PixelFormat.Bgra
							, PixelType.UnsignedByte
							, data.Scan0);

			bmp.UnlockBits(data);

			//텍스처 필터링
			GL.TexParameter(TextureTarget.Texture2D
							, TextureParameterName.TextureWrapS
							, (int)TextureWrapMode.Clamp);
			GL.TexParameter(TextureTarget.Texture2D
							, TextureParameterName.TextureWrapT
							, (int)TextureWrapMode.Clamp);

			GL.TexParameter(TextureTarget.Texture2D
							, TextureParameterName.TextureMinFilter
							, (int)TextureMinFilter.Linear);
			GL.TexParameter(TextureTarget.Texture2D
							, TextureParameterName.TextureMagFilter
							, (int)TextureMagFilter.Linear);

			//텍스처의 정보를 저장하고 정보를 리턴 합니다.
			return new Texture2D(id, bmp.Width, bmp.Height);

		}
	}
}
