﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using OpenTK;
using OpenTK.Graphics.OpenGL;
using System.Drawing;

namespace OpenTK_Test
{
	public enum TweenType
	{
		Instant,
		Linear,
		QuadraticInOut,
		CubicInOut,
		QuarticOut,
	}

	class View
	{
		/// <summary>
		/// 위치
		/// </summary>
		private Vector2 position;
		/// <summary>
		/// 각도, + = 시계방향
		/// </summary>
		public double rotation;
		/// <summary>
		/// 확대
		/// 1 = 기본
		/// 2 = 2x 확대
		/// </summary>
		public double zoom;

		private Vector2 positionGoto, positionFrom;
		private TweenType tweenType;
		private int currentStep, tweenSteps;


		public Vector2 Position
		{
			get
			{
				return this.position;
			}
		}

		public Vector2 ToWorld(Vector2 input)
		{
			input /= (float)zoom;
			Vector2 dX = new Vector2((float)Math.Cos(rotation), (float)Math.Sin(rotation));
			Vector2 dY = new Vector2((float)Math.Cos(rotation + MathHelper.PiOver2)
										, (float)Math.Sin(rotation + MathHelper.PiOver2));

			return (this.position + (dX * input.X) + (dY * input.Y));
		}

		public View(Vector2 startPosition, double startZoom, double startRotation)
		{
			this.position = startPosition;
			this.zoom = startZoom;
			this.rotation = startRotation;
		
		}

		public void Update()
		{
			if (this.currentStep < this.tweenSteps)
			{
				switch (this.tweenType)
				{
					case TweenType.Linear:
						this.position = this.positionFrom + (this.positionGoto - this.positionFrom)
										* GetLinear((float)this.currentStep / this.tweenSteps);
						break;

					case TweenType.QuadraticInOut:
						this.position = this.positionFrom + (this.positionGoto - positionFrom)
										* GetQuadraticInOut((float)currentStep / tweenSteps);
						break;

					case TweenType.CubicInOut:
						this.position = this.positionFrom + (this.positionGoto - positionFrom)
										* GetCubicInOut((float)currentStep / tweenSteps);
						break;

					case TweenType.QuarticOut:
						this.position = this.positionFrom + (this.positionGoto - positionFrom)
										* GetQuarticOut((float)currentStep / tweenSteps);
						break;
				}

				this.currentStep++;

			}
			else
			{
				this.position = this.positionGoto;
			}
		}

		public void SetPosition(Vector2 newPosition)
		{
			
			this.position = newPosition;
			this.positionFrom = newPosition;
			this.positionGoto = newPosition;
			this.tweenType = TweenType.Instant;
			this.currentStep = 0;
			this.tweenSteps = 0;
		}
		public void SetPosition(Vector2 newPosition, TweenType type, int numSteps)
		{
			this.positionFrom = this.Position;
			this.position = newPosition;
			this.positionGoto = newPosition;
			this.tweenType = type;
			this.currentStep = 0;
			this.tweenSteps = numSteps;
		}

		public float GetLinear(float t)
		{
			return t;
		}

		public float GetQuadraticInOut(float t)
		{
			return (t * t) / ((2 * t * t) - (2 * t) + 1);
		}

		public float GetCubicInOut(float t)
		{
			return (t * t) / ((3 * t * t) - (3 * t) + 1);
		}

		public float GetQuarticOut(float t)
		{
			return -((t - 1) * (t - 1) * (t - 1) * (t - 1)) + 1;
		}

		public void ApplyTransform()
		{
			Matrix4 transform = Matrix4.Identity;

			//위치를 잡기위해 음수(-)값을 주므로서
			//양수값을 주는 경우 카메라가 오른쪽으로 이동하는 효과를 주게 됩니다.
			transform = Matrix4.Mult(transform, Matrix4.CreateTranslation(-this.position.X, -this.position.Y, 0));
			transform = Matrix4.Mult(transform, Matrix4.CreateRotationZ(-(float)this.rotation));
			transform = Matrix4.Mult(transform, Matrix4.CreateScale((float)this.zoom, (float)zoom, 1.0f));

			GL.MultMatrix(ref transform);
		}

	}
}
